@@ -12,7 +12,8 @@
 #' together and error estimation/ accuracy determined. The default is 10x10
 #' nested CV.
 #'
-#' @param y Response vector
+#' @param y Response vector or matrix. Matrix is only used for 
+#' `family = 'mgaussian'` or `'cox'`.
 #' @param x Matrix of predictors. Dataframes will be coerced to a matrix as
 #'   is necessary for glmnet.
 #' @param family Either a character string representing one of the built-in
@@ -188,7 +189,7 @@ nestcv.glmnet <- function(y, x,
   x <- as.matrix(x)
   if (is.null(colnames(x))) colnames(x) <- paste0("V", seq_len(ncol(x)))
   ok <- checkxy(y, x, na.option, weights)
-  y <- y[ok$r]
+  y <- if (is.matrix(y)) y[ok$r, ] else y[ok$r]
   x <- x[ok$r, ok$c]
   weights <- weights[ok$r]
   if (!is.null(balance) & !is.null(weights)) {
@@ -197,9 +198,10 @@ nestcv.glmnet <- function(y, x,
     stop("`balance` can only be used for classification")}
   
   if (is.null(outer_folds)) {
+    y1 <- if (is.matrix(y)) y[,1] else y
     outer_folds <- switch(outer_method,
-                          cv = createFolds(y, k = n_outer_folds),
-                          LOOCV = 1:length(y))
+                          cv = createFolds(y1, k = n_outer_folds),
+                          LOOCV = 1:NROW(y))
   } else {
     if ("n_outer_folds" %in% names(nestcv.call)) {
       if (n_outer_folds != length(outer_folds)) stop("Mismatch between n_outer_folds and length(outer_folds)")
@@ -260,7 +262,7 @@ nestcv.glmnet <- function(y, x,
   if (!is.null(rownames(x))) {
     rownames(output) <- unlist(lapply(predslist, rownames))}
   
-  summary <- predSummary(output)
+  summary <- predSummary(output, family = family)
   glmnet.roc <- NULL
   if (family == "binomial") {
     glmnet.roc <- pROC::roc(output$testy, output$predyp, direction = "<", 
@@ -283,7 +285,7 @@ nestcv.glmnet <- function(y, x,
       foldid <- NULL
       if (pass_outer_folds) {
         if (n_outer_folds == n_inner_folds && is.null(balance)) {
-          foldid <- rep(0, length(y))
+          foldid <- rep(0, NROW(y))
           for (i in 1:length(outer_folds)) {
             foldid[outer_folds[[i]]] <- i
           }
@@ -374,8 +376,20 @@ nestcv.glmnetCore <- function(i, y, x, outer_folds, filterFUN, filter_options,
   s <- exp((log(alphafit$lambda.min) * (1-min_1se) + log(alphafit$lambda.1se) * min_1se))
   cf <- glmnet_coefs(alphafit, s = s)
   # test on outer CV
-  predy <- as.vector(predict(alphafit, newx = filt_xtest, s = s, type = "class"))
-  preds <- data.frame(testy=ytest, predy=predy)
+  if (family == "mgaussian") {
+    # mgaussian
+    predy <- predict(alphafit, newx = filt_xtest, s = s)[,, 1]
+    preds <- as.data.frame(cbind(ytest, predy))
+    colnames(preds)[1:ncol(y)] <- paste0("ytest.", colnames(ytest))
+  } else if (family == "cox") {
+    # cox
+    predy <- as.vector(predict(alphafit, newx = filt_xtest, s = s))
+    preds <- as.data.frame(cbind(ytest, predy))
+  } else {
+    # default
+    predy <- as.vector(predict(alphafit, newx = filt_xtest, s = s, type = "class"))
+    preds <- data.frame(testy=ytest, predy=predy)
+  }
   if (family == "binomial") {
     predyp <- as.vector(predict(alphafit, newx = filt_xtest, s = s))
     preds <- cbind(preds, predyp)
@@ -385,8 +399,17 @@ nestcv.glmnetCore <- function(i, y, x, outer_folds, filterFUN, filter_options,
     preds <- cbind(preds, predyp)
   }
   if (outer_train_predict) {
-    train_predy <- as.vector(predict(alphafit, newx = filt_xtrain, s = s, type = "class"))
-    train_preds <- data.frame(ytrain=ytrain, predy=train_predy)
+    if (is.atomic(y)) {
+      train_predy <- as.vector(predict(alphafit, newx = filt_xtrain, s = s, type = "class"))
+      train_preds <- data.frame(ytrain=ytrain, predy=train_predy)
+    } else {
+      # mgaussian, cox
+      train_predy <- predict(alphafit, newx = filt_xtrain, s = s)
+      train_preds <- as.data.frame(cbind(ytrain, train_predy))
+      if (family == "mgaussian") {
+        colnames(train_preds)[1:ncol(y)] <- paste0("ytrain.", colnames(ytrain))
+      }
+    }
     if (family == "binomial") {
       predyp <- as.vector(predict(alphafit, newx = filt_xtrain, s = s))
       train_preds <- cbind(train_preds, predyp)
@@ -408,7 +431,7 @@ nestcv.glmnetCore <- function(i, y, x, outer_folds, filterFUN, filter_options,
   # inner CV predictions
   if (keep) {
     ind <- alphafit$index["min", ]
-    innerCV_preds <- if (family == "multinomial") {
+    innerCV_preds <- if (length(dim(alphafit$fit.preval)) == 3) {
       alphafit$fit.preval[, , ind]
     } else alphafit$fit.preval[, ind]
     ret <- append(ret, list(innerCV_preds = innerCV_preds))
@@ -419,234 +442,3 @@ nestcv.glmnetCore <- function(i, y, x, outer_folds, filterFUN, filter_options,
   }
   ret
 }
-
-
-#' Cross-validation of alpha for glmnet
-#' 
-#' Performs k-fold cross-validation for glmnet, including alpha mixing parameter.
-#' 
-#' @param x Matrix of predictors
-#' @param y Response vector
-#' @param nfolds Number of folds (default 10)
-#' @param alphaSet Sequence of alpha values to cross-validate
-#' @param foldid Optional vector of values between 1 and `nfolds` identifying
-#'   what fold each observation is in.
-#' @param ... Other arguments passed to [cv.glmnet]
-#' @return Object of S3 class "cva.glmnet", which is a list of the cv.glmnet 
-#' objects for each value of alpha and `alphaSet`.
-#' \item{fits}{List of fitted [cv.glmnet] objects}
-#' \item{alphaSet}{Sequence of alpha values used}
-#' \item{alpha_cvm}{The mean cross-validated error - a vector of length 
-#' `length(alphaSet)`.}
-#' \item{best_alpha}{Value of alpha giving lowest `alpha_cvm`.}
-#' \item{which_alpha}{Index of `alphaSet` with lowest `alpha_cvm`}
-#' @seealso [cv.glmnet], [glmnet]
-#' @author Myles Lewis
-#' @importFrom glmnet cv.glmnet
-#' @importFrom utils tail
-#' @export
-#' 
-cva.glmnet <- function(x, y, nfolds = 10, alphaSet = seq(0.1, 1, 0.1),
-                       foldid = NULL, ...) {
-  if (is.null(foldid)) {
-    foldid <- sample(rep(seq_len(nfolds), length = length(y)))
-  }
-  fit1 <- cv.glmnet(x = x, y = y, 
-                    alpha = tail(alphaSet, 1), foldid = foldid, ...)
-  if (length(alphaSet) > 1) {
-    fits <- lapply(alphaSet[1:(length(alphaSet)-1)], function(alpha) {
-      cv.glmnet(x = x, y = y, 
-                alpha = alpha, foldid = foldid, lambda = fit1$lambda, ...)
-    })
-    fits <- append(fits, list(fit1))
-  } else fits <- list(fit1)
-  alpha_cvm <- unlist(lapply(fits, function(i) min(i$cvm)))
-  which_alpha <- which.min(alpha_cvm)
-  best_alpha <- alphaSet[which_alpha]
-  cvafit <- list(fits = fits,
-                  alphaSet = alphaSet,
-                  alpha_cvm = alpha_cvm,
-                  best_alpha = best_alpha,
-                  which_alpha = which_alpha)
-  class(cvafit) <- "cva.glmnet"
-  cvafit
-}
-
-
-#' Extract coefficients from a cva.glmnet object
-#' 
-#' Extracts model coefficients from a fitted [cva.glmnet()] object.
-#' 
-#' @param object Fitted `cva.glmnet` object.
-#' @param ... Other arguments passed to `coef.glmnet()` e.g. `s` the value of
-#'   lambda at which coefficients are required.
-#' @export
-coef.cva.glmnet <- function(object, ...) {
-  coef(object$fits[[object$which_alpha]], ...)
-}
-
-
-#' glmnet coefficients
-#' 
-#' Convenience function for retrieving coefficients from a [cv.glmnet] model at 
-#' a specified lambda. Sparsity is removed and non-intercept coefficients are 
-#' ranked by absolute value.
-#' 
-#' @param fit A [cv.glmnet] fitted model object.
-#' @param s Value of lambda. See [coef.glmnet] and [predict.cv.glmnet]
-#' @param ... Other arguments passed to [coef.glmnet]
-#' @return Vector or list of coefficients ordered with the intercept first, 
-#' followed by highest absolute value to lowest.
-#' @importFrom stats coef
-#' @export
-#' 
-glmnet_coefs <- function(fit, s, ...) {
-  if (length(fit) == 1 && is.na(fit)) return(NA)
-  cf <- coef(fit, s = s, ...)
-  if (is.list(cf)) {
-    cf <- lapply(cf, function(i) {
-      cf <- as.matrix(i)
-      cf <- cf[cf != 0, ]
-      cf2 <- cf[-1]
-      cf2 <- cf2[order(abs(cf2), decreasing = TRUE)]
-      c(cf[1], cf2)
-    })
-    return(cf)
-  } 
-  cf <- as.matrix(cf)
-  cf <- cf[cf != 0, ]
-  cf2 <- cf[-1]
-  cf2 <- cf2[order(abs(cf2), decreasing = TRUE)]
-  c(cf[1], cf2)  # keep intercept first
-}
-
-
-#' Extract coefficients from nestcv.glmnet object
-#' 
-#' Extracts coefficients from the final fit of a `"nestcv.glmnet"` object.
-#' 
-#' @param object Object of class `"nestcv.glmnet"`
-#' @param s Value of penalty parameter lambda. Default is the mean of lambda 
-#' values selected across each outer fold.
-#' @param ... Other arguments passed to [coef.glmnet]
-#' @return Vector or list of coefficients ordered with the intercept first, 
-#' followed by highest absolute value to lowest.
-#' @export
-#'
-coef.nestcv.glmnet <- function(object, s = object$final_param["lambda"], ...) {
-  glmnet_coefs(object$final_fit, s = s, ...)
-}
-
-
-#' @export
-print.nestcv.glmnet <- function(x, digits = max(3L, getOption("digits") - 3L), ...) {
-  cat("Nested cross-validation with glmnet\n")
-  if (!is.null(x$call$filterFUN)) 
-    cat("Filter: ", x$call$filterFUN, "\n") else cat("No filter\n")
-  cat("\nFinal parameters:\n")
-  if (length(x$final_param)==1 && is.na(x$final_param)) {
-    cat("NA\n")
-  } else print(x$final_param, digits = digits, print.gap = 2L)
-  cat("\nFinal coefficients:\n")
-  if (length(x$final_fit)==1 && is.na(x$final_fit)) {
-    cat("NA\n")
-  } else print(coef(x), digits = digits)
-  cat("\nResult:\n")
-  print(x$summary, digits = digits, print.gap = 2L)
-}
-
-
-#' @export
-summary.nestcv.glmnet <- function(object, digits = max(3L, getOption("digits") - 3L), ...) {
-  cat("Nested cross-validation with glmnet\n")
-  if (!is.null(object$call$filterFUN)) 
-    cat("Filter: ", object$call$filterFUN, "\n") else cat("No filter\n")
-  cat("Outer loop: ", switch(object$outer_method,
-                         cv = paste0(length(object$outer_folds), "-fold CV"),
-                         LOOCV = "leave-one-out CV"))
-  cat("\nInner loop: ", paste0(object$n_inner_folds, "-fold CV\n"))
-  balance <- object$call$balance
-  if (!is.null(balance)) {
-    cat("Balancing: ", balance, "\n")
-  }
-  cat(object$dimx[1], "observations,", object$dimx[2], "predictors\n")
-  if (!is.numeric(object$y)) print(c(table(object$y)))
-  cat("\n")
-  
-  alpha <- unlist(lapply(object$outer_result, '[[', 'alpha'))
-  lambda <- unlist(lapply(object$outer_result, '[[', 'lambda'))
-  nfilter <- unlist(lapply(object$outer_result, '[[', 'nfilter'))
-  foldres <- data.frame(alpha = alpha, lambda = lambda, n.filter = nfilter,
-                        row.names = paste("Fold", seq_along(alpha)))
-  
-  print(foldres, digits = digits)
-  cat("\nFinal parameters:\n")
-  if (length(object$final_param)==1 && is.na(object$final_param)) {
-    cat("NA\n")
-  } else print(object$final_param, digits = digits, print.gap = 2L)
-  cat("\nFinal coefficients:\n")
-  if (length(object$final_fit)==1 && is.na(object$final_fit)) {
-    cat("NA\n")
-  } else print(coef(object), digits = digits)
-  cat("\nResult:\n")
-  print(object$summary, digits = digits, print.gap = 3L)
-  out <- list(dimx = object$dimx, folds = foldres,
-              final_param = object$final_param,
-              coef = coef(object), result = object$summary)
-  invisible(out)
-}
-
-#' Predict method for nestcv.glmnet fits
-#'
-#' Obtains predictions from the final fitted model from a [nestcv.glmnet]
-#' object.
-#' 
-#' @param object Fitted `nestcv.glmnet` object
-#' @param newdata New data to predict outcome on
-#' @param s Value of lambda for glmnet prediction
-#' @param ... Other arguments passed to `predict.glmnet`.
-#' @return Object returned depends on the `...` argument passed to predict
-#'   method for `glmnet` objects.
-#' @details Checks for missing predictors and if these are sparse (i.e. have
-#'   zero coefficients) columns of 0 are automatically added to enable
-#'   prediction to proceed.
-#' @seealso [glmnet::glmnet]
-#' @method predict nestcv.glmnet
-#' @export
-predict.nestcv.glmnet <- function(object, newdata,
-                                  s = object$final_param["lambda"],
-                                  ...) {
-  newdata <- as.matrix(newdata)
-  newx <- fix_cols(object$final_fit, newdata, s = s)
-  predict(object$final_fit, newx = newx, s = unname(s), ...)
-}
-
-
-# fills in zero coefficent columns if missing
-fix_cols <- function(x, newx, s) {
-  cf <- coef(x, s = s)
-  # check for multinomial
-  if (is.list(cf)) {
-    cf <- do.call(cbind, cf)
-    cf <- as.matrix(cf)
-  }
-  final_vars <- rownames(cf)[-1]
-  cf <- cf[-1,]
-  # if full subset present
-  if (all(final_vars %in% colnames(newx))) return(newx[, final_vars])
-  # some cols missing
-  nz <- if (is.vector(cf)) {
-    cf != 0
-  } else apply(cf, 1, function(i) any(i != 0))  # multinomial
-  nonzeros <- final_vars[nz]
-  if (!all(nonzeros %in% colnames(newx))) {
-    stop("Some non-zero coefficient predictors are missing")}
-  zeros <- final_vars[!nz]
-  if (length(zeros) == 0) return(newx[, final_vars])
-  miss <- zeros[!zeros %in% colnames(newx)] 
-  if (length(miss) == 0) return(newx[, final_vars])
-  mat <- matrix(0, nrow = nrow(newx), ncol = length(miss),
-                dimnames = list(rownames(newx), miss))
-  out <- cbind(newx, mat)
-  out[, final_vars]  # col order seems to matter
-}
